#define XANBOO                                   0
#define BELKIN_VIDEOBUS_II                       1
#define BELKIN_VIDEOBUS                          2
#define BELKIN_USB_VIDEOBUS_II                   3
#define ECHOFX_INTERVIEW_LITE                    4
#define USBGEAR_USBG_V1                          5
#define D_LINK_V100                              6
#define X10_USB_CAMERA                           7
#define HPG_WINTV_LIVE_PAL_BG                    8
#define HPG_WINTV_LIVE_PRO_NTSC_MN               9
#define ZORAN_PMD_NOGATECH                       10
#define NOGATECH_USB_TV_NTSC_FM                  11
#define PNY_USB_TV_NTSC_FM                       12
#define PV_PLAYTV_USB_PRO_PAL_FM                 13
#define ZT_721                                   14
#define HPG_WINTV_NTSC_MN                        15
#define HPG_WINTV_PAL_BG                         16
#define HPG_WINTV_PAL_I                          17
#define HPG_WINTV_PAL_SECAM_L                    18
#define HPG_WINTV_PAL_D_K                        19
#define HPG_WINTV_NTSC_FM                        20
#define HPG_WINTV_PAL_BG_FM                      21
#define HPG_WINTV_PAL_I_FM                       22
#define HPG_WINTV_PAL_D_K_FM                     23
#define HPG_WINTV_PRO_NTSC_MN                    24
#define HPG_WINTV_PRO_NTSC_MN_V2                 25
#define HPG_WINTV_PRO_PAL                        26
#define HPG_WINTV_PRO_NTSC_MN_V3                 27
#define HPG_WINTV_PRO_PAL_BG                     28
#define HPG_WINTV_PRO_PAL_I                      29
#define HPG_WINTV_PRO_PAL_SECAM_L                30
#define HPG_WINTV_PRO_PAL_D_K                    31
#define HPG_WINTV_PRO_PAL_SECAM                  32
#define HPG_WINTV_PRO_PAL_SECAM_V2               33
#define HPG_WINTV_PRO_PAL_BG_V2                  34
#define HPG_WINTV_PRO_PAL_BG_D_K                 35
#define HPG_WINTV_PRO_PAL_I_D_K                  36
#define HPG_WINTV_PRO_NTSC_MN_FM                 37
#define HPG_WINTV_PRO_PAL_BG_FM                  38
#define HPG_WINTV_PRO_PAL_I_FM                   39
#define HPG_WINTV_PRO_PAL_D_K_FM                 40
#define HPG_WINTV_PRO_TEMIC_PAL_FM               41
#define HPG_WINTV_PRO_TEMIC_PAL_BG_FM            42
#define HPG_WINTV_PRO_PAL_FM                     43
#define HPG_WINTV_PRO_NTSC_MN_FM_V2              44
#define CAMTEL_TVB330                            45
#define DIGITAL_VIDEO_CREATOR_I                  46
#define GLOBAL_VILLAGE_GV_007_NTSC               47
#define DAZZLE_DVC_50_REV_1_NTSC                 48
#define DAZZLE_DVC_80_REV_1_PAL                  49
#define DAZZLE_DVC_90_REV_1_SECAM                50
#define ESKAPE_LABS_MYTV2GO                      51
#define PINNA_PCTV_USB_PAL                       52
#define PINNA_PCTV_USB_SECAM                     53
#define PINNA_PCTV_USB_PAL_FM                    54
#define MIRO_PCTV_USB                            55
#define PINNA_PCTV_USB_NTSC_FM                   56
#define PINNA_PCTV_USB_PAL_FM_V2                 57
#define PINNA_PCTV_USB_NTSC_FM_V2                58
#define PINNA_PCTV_USB_PAL_FM_V3                 59
#define PINNA_LINX_VD_IN_CAB_NTSC                60
#define PINNA_LINX_VD_IN_CAB_PAL                 61
#define PINNA_PCTV_BUNGEE_PAL_FM                 62
#define HPG_WINTV                                63
#define PINNA_PCTV_USB_NTSC_FM_V3                64
#define MICROCAM_NTSC                            65
#define MICROCAM_PAL                             66

extern const int usbvision_device_data_size;
